==> pb_common-1.31-py3-none-any.whl: pb_common/client/client.py <==
from attributedict.collections import AttributeDict
from dateutil.parser import parse

from pb_common.client.agreement import Agreements, AGREE_SECTIONS
from pb_common.client.exceptions import ClientError
from pb_common.client.registration import ClientRegistration
from pb_common.utils import support
from pb_common.utils.logger import get_logger

__author__ = "Maksim Senchuk"

logger = get_logger(__name__)


class Client(AttributeDict):
    """
    Объект - клиент
    """
    def __init__(self, context, client, agreements):
        """
        :param context: Context
        :param client: клиентские данные из Бендера
        :param agreements: список договоров из Бендера
        """
        self._context = context
        self._client = client
        self._agreements = agreements

        self._client_info = self._get_client()
        self._check_client()

        self._dashboard = self._get_dashboard()
        self._check_agreements()

        self.registration = ClientRegistration(context)
        self.agreements = Agreements(context, self._dashboard)

        super().__init__(self._client_info)
        self.update(self._client)

    def refresh(self, client=False, agreements=False, clear_cache=False):
        """
        Получить обновленные клиентские данные
        """
        if clear_cache:
            self._context.client_service.update_cache(path=dict(value=self.integration_id))
            self._context.contract_service.delete_contracts(
                headers=dict(Authorization=f'IntegrationId {self.integration_id}'))
        if client:
            self._client_info = self._get_client()
            self.update(self._client_info)
        if agreements:
            self._dashboard = self._get_dashboard()
            self.agreements = Agreements(self._context, self._dashboard)
        return self

    def _check_client(self):
        """
        Сверить данные клиента из Бендера и Клиент Сервиса
        """
        at = self._client.get('mobile')
        srv = self._client_info.get('contact', {}).get('phoneNumber')
        self._check_data(at, srv, 'Номер телефона')

        at = self._client.get('access_code')
        srv = self._client_info.get('accessCode')
        self._check_data(at, srv, 'Access Code')

        srv = self._client_info.get('contact', {}).get('birthDate')
        at = self._client.get('birth_date')
        self._check_data(at, srv, 'дата рождения')

    def _check_agreements(self):
        """
        Сверить данные договоров клиента из Бендера и Дашборд Сервиса
        """
        dashboard_agreements = []
        for key, value in self._dashboard.items():
            if key in AGREE_SECTIONS:
                if not value:
                    continue
                if isinstance(value, list):
                    dashboard_agreements.extend(value)
                elif isinstance(value, dict):
                    dashboard_agreements.append(value)
                else:
                    raise ValueError('Неожиданный тип договора')

        for agr in self._agreements:
            logger.debug(f"Проверяем договор: {agr['account_id']}")
            for d_agr in dashboard_agreements:
                if str(agr['account_id']) == str(d_agr['id']):
                    logger.debug('Найден!')
                    break
            else:
                ClientError('Не совпадают данные договоров подготовленного клиента и ответа сервиса')

    @support.retry_request(delay=5.0, exceptions=(ClientError, ))
    def _get_client(self):
        """
        Получить информацию по клиенту из ClientService методом GET internal/api/v2/client/{valueId}?type={typeName}
        :return:
        """
        r = self._context.client_service.get_client(path=dict(valueId=self.integration_id, typeName='integrationId'))
        if r.status_code == 200:
            return support.to_attribute_dict(r.json())
        else:
            raise ClientError('Не удалось получить данные по клиенту в Client Service: {}'.format(r.text))

    @support.retry_request(delay=5.0, exceptions=(ClientError, ))
    def _get_dashboard(self):
        """
        Получить дашборд клиента из DashboardService методом GET /dashboardservice/api/dashboard
        :return:
        """
        r = self._context.dashboard_service.get_dashboard(
            headers={'Authorization': self.integration_id, 'x-channel': 'MB'})
        if r.status_code == 200:
            return support.to_attribute_dict(r.json())
        else:
            raise ClientError(
                'Не удалось получить данные из Dashboard Service: {}'.format(r.text))

    @staticmethod
    def _check_data(ref, exc, title):
        """Сравнить данные из Бенедера и сервиса"""
        if ref != exc:
            raise ClientError('Не совпадают данные подготовленного клиента и ответа сервиса. '
                              'Ожидаемый {} - {} , вернул сервис - {}'.format(title, ref, exc))

    @property
    def integration_id(self):
        """
        Вернуть integration_id клиента
        :return: integration_id
        """
        try:
            return self._client['integration_id']
        except KeyError:
            ClientError('Не найден integration_id клиента')

    @property
    def mobile(self):
        """
        Вернуть номер мобильного телефона
        :return: номер в формате +71234567890
        """
        return self._client_info.get('contact', {}).get('phoneNumber')

    @property
    def access_code(self):
        """
        Вернуть код доступа (для регистрации)
        :return: код доступа
        """
        return self._client_info.get('accessCode')

    @property
    def name_f(self):
        """
        Получить имя фамилию клиента в формате: Имя Ф
        :return: Имя Ф
        """
        if self.first_name is not None and self.last_name is not None:
            return '{} {}.'.format(self.first_name.strip(), self.last_name.strip()[0]).title()
        return ''

    @property
    def name_full(self):
        """
        Получить имя фамилию клиента в формате: Фамилия Имя Отчество
        :return: Фамилия Имя Отчество
        """
        if self.first_name is not None and self.last_name is not None:
            if self.middle_name:
                n = '{} {} {}'.format(self.last_name.strip(), self.first_name.strip(), self.middle_name.strip())
            else:
                n = '{} {}'.format(self.last_name.strip(), self.first_name.strip())
            return n.title()
        return ''

    def get_birth_date(self, format_date='%d.%m.%Y'):
        """
        Получить дату рождения клиента в определнном формате
        :param format_date: формат даты
        :return: дата рождения
        """
        if self.birth_date is not None:
            birth_dt = parse(self.birth_date)
            return birth_dt.strftime(format_date)

==> pb_common-1.39-py3-none-any.whl: pb_common/client/client.py <==
import time

from attributedict.collections import AttributeDict
from dateutil.parser import parse

from pb_common.client.agreement import Agreements, AGREE_SECTIONS
from pb_common.client.exceptions import ClientError
from pb_common.client.registration import ClientRegistration
from pb_common.utils import support
from pb_common.utils.logger import get_logger

__author__ = "Maksim Senchuk"

logger = get_logger(__name__)


class Client(AttributeDict):
    """
    Объект - клиент
    """
    def __init__(self, context, client, agreements=None):
        """
        :param context: Context
        :param client: клиентские данные из Бендера
        :param agreements: список договоров из Бендера
        """
        self._context = context
        self._client = client
        self._agreements = [] if not agreements else agreements
        self._auth_token = {}

        self._client_info = self._get_client()
        self._check_client()

        self._dashboard = self._get_dashboard()
        self._check_agreements()

        self.registration = ClientRegistration(context)
        self.agreements = Agreements(context, self._dashboard, self.get_auth_header)

        super().__init__(self._client_info)
        self.update(self._client)

    def refresh(self, client=False, agreements=False, clear_cache=False):
        """
        Получить обновленные клиентские данные
        """
        if clear_cache:
            self._context.client_service.update_cache(path=dict(value=self.integration_id))
            self._context.contract_service.delete_contracts(
                headers=self.get_auth_header(internal=True))
        if client:
            self._client_info = self._get_client()
            self.update(self._client_info)
        if agreements:
            self._dashboard = self._get_dashboard()
            self.agreements = Agreements(self._context, self._dashboard, self.get_auth_header)
        return self

    def _check_client(self):
        """
        Сверить данные клиента из Бендера и Клиент Сервиса
        """
        # для прода не проверять
        if 'prod' in self._context.env.lower():
            return

        at = self._client.get('mobile')
        srv = self._client_info.get('contact', {}).get('phoneNumber')
        self._check_data(at, srv, 'Номер телефона')

        at = self._client.get('access_code')
        srv = self._client_info.get('accessCode')
        self._check_data(at, srv, 'Access Code')

        srv = self._client_info.get('contact', {}).get('birthDate')
        at = self._client.get('birth_date')
        self._check_data(at, srv, 'дата рождения')

    def _check_agreements(self):
        """
        Сверить данные договоров клиента из Бендера и Дашборд Сервиса
        """
        # для прода не проверять
        if 'prod' in self._context.env.lower():
            return

        dashboard_agreements = []
        for key, value in self._dashboard.items():
            if key in AGREE_SECTIONS:
                if not value:
                    continue
                if isinstance(value, list):
                    dashboard_agreements.extend(value)
                elif isinstance(value, dict):
                    dashboard_agreements.append(value)
                else:
                    raise ValueError('Неожиданный тип договора')

        for agr in self._agreements:
            logger.debug(f"Проверяем договор: {agr['account_id']}")
            for d_agr in dashboard_agreements:
                if str(agr['account_id']) == str(d_agr['id']):
                    logger.debug('Найден!')
                    break
            else:
                raise ClientError('Не совпадают данные договоров подготовленного клиента и ответа сервиса')

    @support.retry_request(delay=5.0, exceptions=(ClientError, ))
    def _get_client(self):
        """
        Получить информацию по клиенту из ClientService методом GET internal/api/v2/client/{valueId}?type={typeName}
        :return:
        """
        r = self._context.client_service.get_client(path=dict(valueId=self.integration_id, typeName='integrationId'))
        if r.status_code == 200:
            return support.to_attribute_dict(r.json())
        else:
            raise ClientError('Не удалось получить данные по клиенту в Client Service: {}'.format(r.text))

    @support.retry_request(delay=5.0, exceptions=(ClientError, ))
    def _get_dashboard(self):
        """
        Получить дашборд клиента из DashboardService методом GET /dashboardservice/api/dashboard
        :return:
        """
        r = self._context.dashboard_service.get_dashboard(
            headers={**self.get_auth_header(), 'x-channel': 'MB'})
        if r.status_code == 200:
            return support.to_attribute_dict(r.json())
        else:
            raise ClientError(
                'Не удалось получить данные из Dashboard Service: {}'.format(r.text))

    @staticmethod
    def _check_data(ref, exc, title):
        """Сравнить данные из Бенедера и сервиса"""
        if ref != exc:
            raise ClientError('Не совпадают данные подготовленного клиента и ответа сервиса. '
                              'Ожидаемый {} - {} , вернул сервис - {}'.format(title, ref, exc))

    @property
    def integration_id(self):
        """
        Вернуть integration_id клиента
        :return: integration_id
        """
        try:
            return self._client['integration_id']
        except KeyError:
            raise ClientError('Не найден integration_id клиента')

    @property
    def mobile(self):
        """
        Вернуть номер мобильного телефона
        :return: номер в формате +71234567890
        """
        return self._client_info.get('contact', {}).get('phoneNumber')

    @property
    def access_code(self):
        """
        Вернуть код доступа (для регистрации)
        :return: код доступа
        """
        return self._client_info.get('accessCode')

    @property
    def name_f(self):
        """
        Получить имя фамилию клиента в формате: Имя Ф
        :return: Имя Ф
        """
        if self.first_name is not None and self.last_name is not None:
            return '{} {}.'.format(self.first_name.strip(), self.last_name.strip()[0]).title()
        return ''

    @property
    def name_full(self):
        """
        Получить имя фамилию клиента в формате: Фамилия Имя Отчество
        :return: Фамилия Имя Отчество
        """
        if self.first_name is not None and self.last_name is not None:
            if self.middle_name:
                n = '{} {} {}'.format(self.last_name.strip(), self.first_name.strip(), self.middle_name.strip())
            else:
                n = '{} {}'.format(self.last_name.strip(), self.first_name.strip())
            return n.title()
        return ''

    def get_birth_date(self, format_date='%d.%m.%Y'):
        """
        Получить дату рождения клиента в определнном формате
        :param format_date: формат даты
        :return: дата рождения
        """
        if self.birth_date is not None:
            birth_dt = parse(self.birth_date)
            return birth_dt.strftime(format_date)

    @property
    def auth_token(self):
        """
        Вернуть авторизационный токен клиента
        :return: токен
        """
        if not self._auth_token or time.time() < self._auth_token['expired']:
            self._auth_token.update(self.registration.get_token(self.integration_id))
        return self._auth_token['token']

    def get_auth_header(self, internal=False):
        """
        Вернуть авторизационный хэдер
        :param internal: тип запроса
        :return: хэдер
        """
        if internal:
            return {'Authorization': f'IntegrationId {self.integration_id}'}
        elif 'prod' in self._context.env.lower():
            return {'Authorization': self.auth_token}
        else:
            return {'Authorization': self.integration_id}
